import openpyxl
import pandas as pd
import numpy as np
import os
import json
import re
import warnings
import time
from statistics import mean
from pathlib import Path
from script.misc import *

warnings.simplefilter("ignore")

class Calculate:
    def __init__(self):
        self.statics = self.read_json()
        self.static_tables = self.statics['tables']

        self.frames = self.statics['frames']

        self.wells = self.statics['wells']
        self.well_names = [chr(i) for i in range(65, 65 + self.wells)]
        self.valid_wells = self.statics['valid_wells']

        self.ROOT_DIR = Path(__file__).parent.parent
        self.out_folder = self.ROOT_DIR / self.statics['out_folder']

        self.temp_dict = {}
        self.avg_df_dict = {}

        self.DEBUG = False
    
    def step2fileread(self, dir):
        self.dir_step2 = dir
        self.files1 = os.listdir(dir)
        
        print('\n', '#' * 10, 'FILES FOR STEP 2 FOUND', '#' * 10,'\n')
        for index, files in enumerate(self.files1):
            print(f'{index} -) {files}')
        print()
        
    def batchfileread(self, dir):
        self.dir_batch = dir
        self.files2 = os.listdir(dir)
        
        print('\n', '#' * 10, 'FILES FOR BATCH FOUND', '#' * 10,'\n')
        for index, files in enumerate(self.files2):
            print(f'{index} -) {files}')
        print()
        
    def read_json(self):
        with open('settings/statics.json', 'r') as file:
            data = file.read()

        return json.loads(data)
        
    def get_samples(self, df):
    
        #print(df.columns)
        locs = df.apply(lambda xx: xx.count(), axis=1)
        n_tot = len(locs)
        nx = locs[locs == 0]
        n_10 = nx.index[0] if len(nx) > 0 else n_tot
        #print(n_10)
        
        df_input = df.loc[:,[i for i in df.columns if i.startswith("Average")]]
        df_input = df_input.iloc[:n_10,1:].div(df_input.iloc[:n_10,0], axis=0)
         
        #df_input = df.iloc[n_l0:n_tot,:].dropna(how='all', axis=1)
        #print(df_input)
        return df_input
    
    def colname_s2(self, i):
        return 'Count' if i < 1 else 'Count.{}'.format(i)
    
    def step2(self, stepf):
        well_length = self.frames
        
        ## SHEET
        wb = openpyxl.load_workbook(stepf)
        if len(wb.sheetnames) > 1:
            print("Sheet Names", wb.sheetnames, '\n')
            collist = sorted(list(set(wb.sheetnames)-set(['Control','Summary'])))
        
            tables = {x: pd.read_excel(stepf, sheet_name=x, skiprows=1, engine="openpyxl") for x in collist}
            totals = {'Well-{}'.format(j+1): [tables[x][self.colname_s2(i)].notna().sum() for i in range(well_length)] 
                        for j, x in enumerate(tables.keys())}
        
        else:
            table = pd.read_excel(stepf, skiprows=1, engine="openpyxl")
            # get max column
            max_col = table.shape[1]
            print('Total columns: ', max_col, '\n')
            limits = []
            for col in range(0, max_col, 30):
                limits.append((col, col+29))
            print('Limits: ', limits, '\n')
            # limits = [(i*30, (i+1)*30 - 1) for i in range(6)]
            
            tables = {chr(65+i): table.iloc[:,l:h] for i, (l, h) in enumerate(limits)}
            totals = {'Well-{}'.format(j+1): [tables[x][self.colname_s2(i)].notna().sum() for i in range(j*well_length, (j+1)*well_length)] 
                        for j, x in enumerate(tables.keys())}
            
            #A0 = len(cstep01.iloc[:, :29]['Count'].dropna())
            
        ## self.wells
        self.wells = pd.DataFrame.from_dict(totals, orient='index', columns = ['Frame {}'.format(i+1) for i in range(well_length)])
        means = self.wells.mean(axis=1)
        self.wells['Average'] = means
        print("STEP 2 SUMMARY\n",  '-'*15+'\n', self.wells.to_string(), '\n', sep='')
        #print('\n', '#' * 10 + ' Step-2 Mean ' + '#' * 10, '\n')
        #print(*['Well-{}: {}\n'.format(i+1, mu) for i, mu in enumerate(means)], sep='')
        
        return (means, self.wells, tables)
    
    def batch(self, batch_filename, sheetname, control_average):
        Element = pd.read_excel(batch_filename, sheet_name = sheetname, engine="openpyxl", header=0)
        Element_input = self.get_samples(Element)
        
        Element_result = []
        
        nEl = len(Element_input)
        nElc = len(Element_input.columns)
        
        for i_1 in range(nEl):
            tmp_column = []
            for j_1 in range(nElc):
                tmp_column.append(
                    (control_average[j_1] - Element_input.iloc[i_1].iloc[j_1]) / (control_average[j_1])
                )
            Element_result.append(tmp_column)

        return pd.DataFrame(Element_result)
    
    def get_dayresult(self, step2_filename):
        summary, _, _ = self.step2(step2_filename)
               
        results = []
        for key, value in self.static_tables.items():
            result = [a / b for a, b in zip(value, summary)]
            results.append(result)

        print('\n', '#' * 10, 'DAY RESULTS', '#' * 10,'\n')
        day_result = []
        for i, result in enumerate(results):
            i += 1
            print(f'Day {i} >> {mean(result)}')
            day_result.append(mean(result))
            
        return (day_result, summary)
    
    def batchproc(self, batch_filename):
    
        results = {}
        wb = openpyxl.load_workbook(batch_filename)
        print("Sheet Names", wb.sheetnames, '\n')
              
        ## CONTROL
        control = pd.read_excel(batch_filename, sheet_name='Control', header=0, engine="openpyxl")
        control_input = self.get_samples(control)

        control_average = []
        control_result = []
        number_cols = len(control_input.columns)
        number_rows = len(control_input)

        for i in range(number_cols):
            tmp_row = []
            for j in range(number_rows):
                tmp_row.append(control_input.iloc[j].iloc[i])
            control_average.append(mean(tmp_row))
            
        print("CONTROL AVERAGES\n", '-'*15+'\n', *["Day {}\t{}\n".format(di+1, control_average[di]) for di in range(len(control_average))], sep='')
        results = {'Control': pd.DataFrame(control_average)}
        
        
        ## OTHER SHEETS
        collist = list(set(wb.sheetnames)-set(['Control','Calc']))
        for x in collist:
            second_data = self.batch(batch_filename, x, control_average)
                        
            print("{} RESULT\n".format(x),  '-'*15+'\n', second_data.to_string(), '\n', sep='')
            results[x] = second_data.copy()

        print('Calculation is done.')
        
        return results
       
    
    def step2save(self, results_dict, output_path):
        
        print("Saving:")
        
        for filename, (summary, self.wells, tables) in results_dict.items():
            sheetname = '.'.join(filename.split('.')[:-1]).split('/')[-1]
            
            writer = pd.ExcelWriter(output_path, engine='xlsxwriter')
            print(output_path)
            self.wells.to_excel(writer, 'Summary')  
                        
            for sheet, table in tables.items():
                temp_t = table.rename(columns = { 
                    i: '' if i.startswith("Unnamed") else '.'.join([x for x in i.split('.') if not x.isdigit()]) 
                    for i in table.columns } )
                temp_t.to_excel(writer, sheet, index=False, startrow = 1)
            
            writer.save()
        
        if len(results_dict) > 1:
            print(output_path)
            writer = pd.ExcelWriter(output_path, engine='xlsxwriter')
            slist = []
            for filename, (summary, self.wells, _) in results_dict.items():
                sheetname = '.'.join(filename.split('.')[:-1]).split('/')[-1]
                self.wells.to_excel(writer, sheetname)  
                
                slist.append(summary.rename(sheetname))
            
            pd.concat(slist, axis=1).to_excel(writer, 'Averages')
            writer.save()
        
    def batchsave(self, results_dict, outputfile, outcontrol = False):
        writer = pd.ExcelWriter(outputfile, engine='xlsxwriter')
        print("Saving:")
        print(outputfile)
        
        for file_path, results in results_dict.items():
            file_path = Path(file_path)
            filename = file_path.stem
            sheetn = filename.split('/')[-1]
            # print('sheetn', sheetn)
            filex = sheetn.split('_')
            # print('filex', filex)
            batchprocname = filex[1] if len(filex) >= 2 else 'b'
            # print('batchprocname', batchprocname)

            collist = list(set(results.keys())-set(['Control','Calc']))
            control = results['Control']
            for x in collist:
                sheetname = batchprocname + '_' + x
                # print('sheetname', sheetname)
                temp_res = results[x].copy()
                if outcontrol:
                    temp_res['Control'] = control
                temp_res.to_excel(writer, sheetname)
        
        writer.save()

        print('------------------------------------------')
        print('-------------------DONE-------------------')
        print('------------------------------------------')

    
    def mergeinputs1(self, filelist):
        outputfile = self.out_folder + 'step2_in.xlsx'
        writer = pd.ExcelWriter(outputfile, engine='xlsxwriter')
        mergefiles = {file: pd.read_excel(self.dir_step2 + file, skiprows=1, engine="openpyxl") for file in filelist}
            
        for file, table in mergefiles.items():
            sheetname = '.'.join(file.split('.')[:-1])
            table.to_excel(writer, sheetname, index=False)
            
        writer.save()
        
    def runs2(self, filename1):
        ## STEP 2
        #print('\n','-' * 36)
        #print('\n', '#' * 10, 'STEP 2 SELECT', '#' * 10,'\n')
        #for index, files in enumerate(self.files1):
        #    print(f'{index} -) {files}')
        #
        #print()
        #print('Choose an excel file: ', end='')
        #file_id = int(input())
        #filename1 = self.dir_step2 + self.files1[file_id]
        
        summary = self.step2(filename1)
        
        #step2 STORING
        output_path = self.out_folder + "step2_" + os.path.basename(filename1) + ".xlsx"
        self.step2save({filename1: summary}, output_path)
    
    def runbatch(self, filename2, outdir = None):
        ## BATCH
        #print('\n','-' * 36)
        #print('\n', '#' * 10, 'BATCH SELECT', '#' * 10,'\n')
        #for index, files in enumerate(self.files2):
        #    print(f'{index} -) {files}')
        #
        #print('\n', 'Choose an excel file: ', end='')
        #file_id = int(input())
        #filename2 = self.dir_batch + self.files2[file_id]
        
        results = self.batchproc(filename2)
        file_name = "Final_Result_" + os.path.basename(filename2)

        #batch STORING
        if outdir == None:
            outputfile = os.path.join(self.out_folder, file_name)
        else:
            outputfile = os.path.join(outdir, file_name)

        #rename if existed
        i = 1
        while True:
            if os.path.exists(outputfile):
                # add number extension to file name
                file_name = "Final_Result_" + os.path.basename(filename2).split('.')[0] + f'_{i}.xlsx'
                outputfile = self.out_folder + file_name
            else:
                break

        self.batchsave({filename2: results}, outputfile)
            
    def runalls2(self):
        ## STEP 2
        #print('\n','-' * 36)
        #print('\n', '#' * 10, 'STEP 2', '#' * 10,'\n')
        
        results1 = {}
        for index, files in enumerate(self.files1):
            print('-' * 36, '\n', f'{index+1} -) {files}\n', '-' * 36, '\n', sep='')
            results1[files] = self.step2(self.dir_step2 + files)
        
        ##step2 STORING
        outputfile = 's2_summary.xlsx'
        self.step2save(results1, self.out_folder, outputfile)
        #self.mergeinputs1(self.files1)
    
    def runallbatch(self):
        ## BATCH
        #print('\n','-' * 36)
        #print('\n', '#' * 10, 'BATCH', '#' * 10,'\n')
        
        results2 = {}
        for index, files in enumerate(self.files2):
            print('-' * 36, '\n', f'{index+1} -) {files}\n', '-' * 36, '\n', sep='')
            results = self.batchproc(self.dir_batch + files)
            results2[files] = results.copy()
        
        #batch STORING
        outputfile = self.out_folder + 'batch.xlsx'
        self.batchsave(results2, outputfile)

    ### VERSION 1.3 ADDED ###
    ### BY VINCENT L ###
    ### 2023 Mar 03 ###
    ### FOR MERGING SUMMARY DATA INTO BATCH INPUT ###

    def load_raw(self, file_path):
        wb = openpyxl.load_workbook(file_path)

        CHECKLISTS = {
            'Has "Working" in first row': False,
            'Data structure is correct': False,
        }

        # count number of sheets
        # if more than 1 sheet, test the sheet with the most columns
        if len(wb.sheetnames) == 1:
            target_sheet = wb[wb.sheetnames[0]]
        if len(wb.sheetnames) > 1:
            # target_sheet would be the one with the most columns
            for sheet in wb.sheetnames:
                if len(wb[sheet].columns) > len(target_sheet.columns):
                    target_sheet = wb[sheet]

        # check if first row has "working" in one of the cells
        if any('working' in cell.value.lower() for cell in target_sheet[1]):
            CHECKLISTS['Has "Working" in first row'] = True

        # skip first row,
        datasheet = pd.read_excel(file_path, sheet_name = target_sheet.title, skiprows=1, engine="openpyxl")
        #  each dataframe is 5 columns, dataself.frames are separated by empty column(s)
        # split datasheet into several dataself.frames, separated by empty column(s)
        tables = {}
        table_index = 0
        temp_df = pd.DataFrame()
        for col in datasheet.columns:
            if self.DEBUG:
                print('Parsing column: ', col, '...')
            if 'Unnamed' in col:
                # remove rows with NaN in temp_df
                temp_df = temp_df.dropna()
                # save temp_df to tables
                tables[str(table_index+1)] = temp_df
                if self.DEBUG:
                    print('Saved to table: ', str(table_index+1), '...')
                table_index += 1
                temp_df = pd.DataFrame()
                if self.DEBUG:
                    print('Reset temp_df...')
                continue
            temp_df[col.split('.')[0]] = datasheet[col]

            if col == datasheet.columns[-1]:
                # remove rows with NaN in temp_df
                temp_df = temp_df.dropna()
                # save temp_df to tables
                tables[str(table_index+1)] = temp_df
                if self.DEBUG:
                    print('Saved to table: ', str(table_index+1), '...')

        # check if each dataframe has 5 columns
        for key in tables.keys():
            if len(tables[key].columns) == 5:
                CHECKLISTS['Data structure is correct'] = True

        ISRAW = True
        ISRAW *= all(CHECKLISTS.values())

        if not ISRAW:
            return ISRAW, None, CHECKLISTS
        
        return ISRAW, tables, CHECKLISTS
    
    def export_summary(self, tables, output_path):
        summary_columns = [f'Frame {i}' for i in range(1, self.frames + 1)]
        summary_index = [f'Well-{i}' for i in range(1, self.wells + 1)]
        summary_df = pd.DataFrame(columns=summary_columns, index=summary_index)

        for well in range(1, self.wells + 1):
            for frame in range(1, self.frames + 1):
                summary_df.loc[f'Well-{well}', f'Frame {frame}'] = tables[str(well*frame)]['Count'].max()

        # create a new column "Average" in summary_df
        # calculate the average of each row
        summary_df['Average'] = summary_df.mean(axis=1)

        append_df_to_excel(output_path, summary_df, sheet_name='Summary', index=True, header=True)

    
    def summary_process(self, file_path, output_path, file_name):
        file_path = Path(file_path)
            
        df = self.step2(file_path)

        output_path = output_path / f'{file_name}.xlsx'

        self.step2save({file_name: df}, output_path)

    def mergefiles(self, BATCH_DIR, name = None):

        # DEFINE SOME FUNCTIONS
        print()
        print("---------------------------------------------")
        print("MERGE PROCESS STARTED...")
        print("---------------------------------------------")

        valid_wells = {}
        for sample in self.samples:
            if sample == 'Control':
                default_value = self.valid_wells['Control']
            else:
                default_value = self.valid_wells['Samples']
            valid_wells[sample] = input_with_type(f'For {sample}, how many valid wells are there in total?', int, default_value = default_value)

        print("MERGING FILES, PLEASE WAIT...")

        # Define OUTPUT_PATH
        if name == None:
            output_path = os.path.join(BATCH_DIR, f'{os.path.basename(BATCH_DIR)}.xlsx')
            if os.path.exists(output_path):
                os.remove(output_path)
        else:
            PARENT = os.path.dirname(BATCH_DIR)
            output_path = os.path.join(PARENT, 'Merged', f'{name}.xlsx')

        # PATHS_DF = pd.DataFrame(columns=[f'day{i}' for i in range(0, self.days)])
        self.days_list = os.listdir(BATCH_DIR)
        PATHS_DF = pd.DataFrame(columns=self.days_list)

        # find all files in the batch directory, any level deep, with .xlsx extension
        for file_path in Path(BATCH_DIR).rglob('*.xlsx'):
            # get file base name without extension
            file_basename = file_path.stem
            file_basename = str(file_basename).upper()
            if not re.search(r'\d', file_basename):
                file_basename = 'Control'
            # find in the file_path, see which day it belongs to
            for day in self.days_list:
                if day in str(file_path):
                    # add the file_path to the dataframe
                    PATHS_DF.loc[file_basename, day] = file_path
                    break

        AVG_DF_DICT = {}
        startcol = 0
        for day in PATHS_DF.columns:
            temp_valid_wells = valid_wells.copy()
            temp_dict = {}
            lastrow = 0
            # interate through index
            for category in PATHS_DF.index:
                file_path = PATHS_DF.loc[category, day]

                # if the file_path is Nan, raise Exception
                if pd.isna(file_path):
                    raise Exception(f'Missing file for {category} on {day}')
                
                try:
                    df = pd.read_excel(file_path, sheet_name='Summary', header=0, engine="openpyxl")
                except ValueError:
                    print(f'{file_path} has no Worksheet with the name Summary')
                    
                # if df is empty (no data), raise Exception
                if df.empty:
                    raise Exception(f'Empty file for {category} on {day}')
                
                string_part, _ = splitter(category)
                if self.DEBUG:
                    print(f'DF of {os.path.basename(file_path)} has {len(df)} wells')
                    print(f'Valid wells has {temp_valid_wells[string_part]} wells remain')
                if temp_valid_wells[string_part] >= len(df):
                    temp_valid_wells[string_part] -= len(df)
                else:
                    limiter = temp_valid_wells[string_part]
                    if self.DEBUG:
                        print(f'Limited to the first {limiter} row of DF')
                    df = df[:limiter]
                
                if self.DEBUG:
                    print()

                # # if index has number 3 (last set of wells), remove the last 3 rows of the df
                # if re.search(r'3$', category):
                #     df = df[:3]

                # if index has no number (Control) or has number 1 (First set of wells) in it, set following inputs
                if not re.search(r'\d', category) or re.search(r'1$', category):
                    startrow = 0
                    header = True
                    temp_dict[remove_num(category)] = df['Average'].tolist()
                else:
                    if re.search(r'2$', category):
                        startrow = lastrow + 1
                    else:
                        startrow = lastrow
                    header = False
                    temp_dict[remove_num(category)].extend(df['Average'].tolist())

                # update lastrow
                lastrow = startrow + len(df)

                # Append the df to the excel file
                sheet_name = get_sheet_name(category)
                append_df_to_excel(output_path, df, sheet_name=sheet_name, startrow = startrow, startcol = startcol, index=False, header = header)

            if day == 'Day_0':
                for key in temp_dict.keys():
                    AVG_DF_DICT[key] = pd.DataFrame(columns=self.days_list)
            for key in temp_dict.keys():
                AVG_DF_DICT[key][day] = temp_dict[key]

            df_width = len(df.columns)
            startcol += df_width

        print('Calculating and adding Inhibition Average to excel files...')
        # append the average df to the excel file
        for key in AVG_DF_DICT.keys():
            append_df_to_excel(output_path, AVG_DF_DICT[key], sheet_name=key, startrow = None, startcol = 1, row_sep = 1, index=False, header = False)


        print('Calculating and adding Inhibition Average % to excel files...')
        # calculate the compare df
        COMPARE_DF_DICT = {}
        for key in AVG_DF_DICT.keys():
            # compare_df = avg_df.iloc[:,1:].div(avg_df.iloc[:,0], axis=0)
            COMPARE_DF_DICT[key] = AVG_DF_DICT[key].iloc[:,1:].div(AVG_DF_DICT[key].iloc[:,0], axis=0)
        
        # append the compare df to the excel file
        for key in COMPARE_DF_DICT.keys():
            append_df_to_excel(output_path, COMPARE_DF_DICT[key], sheet_name=key, startrow = None, startcol = 2, row_sep = 1, index=False, header = False)

        print()
        print('Merged files were created!')
        print()
        time.sleep(2)

        return output_path


    def allinone(self, BATCH_DIR):
        # --------------------------------------------
        print("PROCESSING... PLEASE WAIT...")

        # Define OUTPUT_PATH
        output_path = os.path.join(BATCH_DIR, f'{os.path.basename(BATCH_DIR)}.xlsx')
        if os.path.exists(output_path):
            os.remove(output_path)

        # Get Days list base on the subfolders of BATCH_DIR
        self.days_list = os.listdir(BATCH_DIR)
        # search in each subfolder for .xlsx files, if not found, take that subfolder out of the days_list (Use pathlib)        PATHS_DF = pd.DataFrame(columns=self.days_list)
        batch_dir = Path(BATCH_DIR)
        for day in self.days_list:
            day_path = batch_dir / day
            if not any(day_path.glob('*.xlsx')):
                self.days_list.remove(day)
        # define PATHS_DF with the new days_list
        PATHS_DF = pd.DataFrame(columns=self.days_list)
        print('Days with .xlsx files inside: ', self.days_list)

        # find all files in the batch directory, any level deep, with .xlsx extension
        nname_dict = {}
        for file_path in batch_dir.rglob('*.xlsx'):
            # get file base name without extension
            file_basename = file_path.stem
            print(f'Processing file {file_basename}')
            file_basename = str(file_basename).upper()
            try:
                nname = get_sheet_name(file_basename)
                nname_dict[file_basename] = nname
                print(f'File {file_basename} is saved as {get_sheet_name(file_basename)}')
            except Exception as e:
                print(f'{e}, file {file_basename} type is {type(file_basename)}')
                continue
            # find in the file_path, see which Day_ it belongs to
            for day in self.days_list:
                if day in str(file_path):
                    # add the file_path to the dataframe
                    PATHS_DF.loc[file_basename, day] = file_path
                    break 

        # get the list of samples (Control, Substance1, Substance2, etc.)
        self.samples = list(set(nname_dict.values()))

        # create empty dict to store average dataself.frames
        for nname in self.samples:
            self.avg_df_dict[nname] = pd.DataFrame(columns=self.days_list)

        # Define final output paths
        suffix_num = 1
        OUTPUT_DIR = self.out_folder / f'{batch_dir.name}_run_{str(suffix_num)}'
        while True:
            if OUTPUT_DIR.exists():
                print(f'OUTPUT DIR with the name {OUTPUT_DIR} already existed!')
                OUTPUT_DIR = self.out_folder / f'{batch_dir.name}_run_{str(suffix_num)}'
                print(f'New output dir: {OUTPUT_DIR}')
                suffix_num += 1
            else:
                break
        print(f'Output dir: {OUTPUT_DIR}')
        OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
        

        OUTPUT_PATHS = {
            'Summary': OUTPUT_DIR / 'Summary',
            'Merged': OUTPUT_DIR / 'Merged',
            'Final': OUTPUT_DIR / 'Final'
        }

        for each_path in OUTPUT_PATHS.values():
            each_path.mkdir(parents = True, exist_ok = True)

        # STEP 1: Split raw data into individual self.wells
        wells_count = {}
        for day in self.days_list:
            for file_path in PATHS_DF[day].dropna():
                ISRAW, tables, CHECKLISTS = self.load_raw(file_path)
                if not ISRAW:
                    raise Exception(f'File {file_path} is not a raw data file \n {CHECKLISTS}')
                frame_count = len(tables)
                print(f'{file_path} is a raw data file')
                print(f'Number of frames: {frame_count}')
                if frame_count % self.frames != 0:
                    raise Exception(f'File {file_path} has {frame_count} frames, not a multiple of {self.frames}')
                print(f'Number of wells: {frame_count // self.frames}')
                print()
                output_path = OUTPUT_PATHS['Summary'] / day
                output_path.mkdir(parents=True, exist_ok=True)
                file_basename = file_path.stem
                if file_basename not in wells_count:
                    wells_count[file_basename] = frame_count // self.frames
                else:
                    diff = wells_count[file_basename] - frame_count // self.frames
                    if diff == 0:
                        pass
                    else:
                        raise Exception(f'File {file_basename} has {frame_count // self.frames} wells, not equal to {wells_count[file_basename]}')
                self.summary_process(file_path, output_path, file_basename)                
        
        # STEP 2: MERGE
        name = 'merged'
        for sample in self.samples:
            if sample == 'Control':
                continue
            name += f'_{sample}'

        for file_name in wells_count.keys():
            sample = nname_dict[file_name]
            temp_ingroup = []
            for basename, nname in nname_dict.items():
                if nname == sample:
                    temp_ingroup.append(basename)
            print(f'Sample {sample} have {len(temp_ingroup)} files')
            temp_total = 0
            for file in temp_ingroup:
                print(f'{file} has {wells_count[file]} wells')
                temp_total += wells_count[file]
            print(f'For a total of {temp_total} wells')

        merged_path = self.mergefiles(OUTPUT_PATHS['Summary'], name)

        # STEP 3: ANALYZE
        self.runbatch(merged_path, outdir = OUTPUT_PATHS['Final'])



if __name__ == '__main__':
    calc = Calculate()
    calc.run()
